/*	Renegade Scripts.dll
	ArKa_47 Bank Plugin
	Copyright 2021 ArKa_47, Shadow-Team

	Report errors to info@Shadow-Team.org

	This file is part of the Renegade scripts.dll
	The Renegade scripts.dll is free software; you can redistribute it and/or modify it under
	the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2, or (at your option) any later
	version. See the file COPYING for more details.
	In addition, an exemption is given to allow Run Time Dynamic Linking of this code with any closed source module that does not contain code covered by this licence.
	Only the source code to the module(s) containing the licenced code has to be released.
*/

#include "General.h"
#include "gmplugin.h"
#include "da.h"
#include "da_event.h"
#include "da_chatcommand.h"

#include "da_settings.h"

#include "ArKaBankSystem.h"
#include "ArKaBank.h"

#define ARKA_BANK_VERSION 0.9f
#define ARKBANK_NAME arkabank
 



ArKaBankClass ARKBANK_NAME;

ArKaBankClass::ArKaBankClass() {
	Console_Output("Loading Bank System Plugin by ArKa_47 version %.1f\n", ARKA_BANK_VERSION);

	// Create a new BankList - have to be the done 1st! - Otherwise ChatCommand and Events won't work properly
	bankList = new ArKaBankListClass("arka_bank.tsv");
	
	// Creates the ChatCommand
	Register_Full_Chat_Command(ArKaBankChatCommandClass, "!bank", 1, DAAccessLevel::NONE, DAChatType::TEAM | DAChatType::PRIVATE);

	// Creates the Event
	bankEvent = new ArKaBankEventClass();

	SettingsFileName = "arka_bank.ini";
}


ArKaBankClass::~ArKaBankClass() {
	Console_Output("Un-Loading Bank System Plugin by ArKa_47 version %.1f\n", ARKA_BANK_VERSION);

	DASettingsManager::Remove_Settings(SettingsFileName);

	DAChatCommandManager::Unregister_Chat_Command("!bank");
	delete bankEvent;
	bankEvent = NULL;

	bankList->Save(); //the destructor of BankListClass allready safes into file
	delete  bankList;
	bankList = NULL;
}

ArKaBankListClass* ArKaBankClass::Get_BankList() {
	return this->bankList;
}


//  ********************************************* ArKaBankChatCommand *****************************************************************
// ---------------------- CHAT COMMAND event -----------------------
bool ArKaBankChatCommandClass::Activate(cPlayer* Player, const DATokenClass& Text, TextMessageEnum ChatType) {
	if (Text.Size() == 0) // not really needed , without a param the line  get called
		bankList->Print_Info(Player);
	else{



		float money;
		if (Is_Number(Text[2]))
			Text.As_Float(2, money);
		else
			money = 0;
		

		if (Text[1] == "dp" || Text[1] == "depose") {
			if (Is_Number(Text[2])) {
				bankList->Add_Money(Player, money);
			//	Console_Output("Limit: %.0f", Settings->Get_Float("ArKaBankSystem", "ArKaBVankLimit", 3000) );
			}
			else
				if (Text[2] == "all" || Text[2] == "")
			bankList->Add_Money(Player, Player->Get_Money());
		}
		else{
			if (Text[1] == "wd" || Text[1] == "withdraw") {
				if (Is_Number(Text[2])) {
					bankList->Remove_Money(Player, money);
				}
				else
					if (Text[2] == "all" || Text[2] == "")
				bankList->Remove_Money(Player, bankList->Get_Balance(Player) );
			}
		}
			if (Text[1] == "info") {
					bankList->Print_AccountInfo(Player);
					return false;
				}
				else{
					bankList->Print_Info(Player) ;
					return false;
				}
	}
	return true;
}

// ---------------------- constructor -----------------------------
ArKaBankChatCommandClass::ArKaBankChatCommandClass() {

	this->bankList = ARKBANK_NAME.Get_BankList();
	if (!bankList)
		Console_Output("bankList incorrect loaded in Chat Command. Bank System might not work properly\n");
}

// ---------------------- destructor -----------------------------
ArKaBankChatCommandClass::~ArKaBankChatCommandClass(){
	//this->bankList->Save();
	this->bankList = NULL;
}


// ********************************************* ArKaBankEvent **********************************************************************************


 // ---------------------- constructor -----------------------------
ArKaBankEventClass::ArKaBankEventClass() {
	Register_Event(DAEvent::GAMEOVER, INT_MAX);
	Register_Event(DAEvent::LEVELLOADED, INT_MAX);
	Register_Event(DAEvent::SETTINGSLOADED, INT_MAX);

	SettingsFileName = "arka_bank.ini";
	DASettingsManager::Add_Settings(SettingsFileName);

	/* Get_Settings(const char* Name) crashed my server idk why	*/
	int Pos = 0;
	for (int i = 0; i < DASettingsManager::Get_Settings_Count(); i++) {
		if (*DASettingsManager::Get_Settings(i)->Get_File_Name() == *SettingsFileName)
			Pos = i;
	}
	Settings = DASettingsManager::Get_Settings(Pos);
	
	this->bankList = ARKBANK_NAME.Get_BankList();
	if (!bankList)
		Console_Output("bankList incorrect loaded in Event. Bank System might not work properly\n");
}

// ---------------------- destructor -----------------------------
ArKaBankEventClass::~ArKaBankEventClass(){
	Unregister_Event(DAEvent::GAMEOVER);
	Unregister_Event(DAEvent::LEVELLOADED);
	Unregister_Event(DAEvent::SETTINGSLOADED);

	DASettingsManager::Remove_Settings(SettingsFileName);

	this->bankList = NULL;
}

// ---------------------- GAME OVER event -----------------------------
void ArKaBankEventClass::Game_Over_Event(){
	if (bankList)
	bankList->Save();
}

// ---------------------- LEVEL LOADED event -----------------------------
void ArKaBankEventClass::Level_Loaded_Event(){
	if (bankList)
		bankList->Load();

	this->Load_settings();
}

// ---------------------- SETTINGS LOADED event -----------------------------
void ArKaBankEventClass::Settings_Loaded_Event(){
	this->Load_settings();
}

// --------------------- Load Settings from ini file -------------------------
void ArKaBankEventClass::Load_settings(){
	bankList->Set_MaxAmount(Settings->Get_Float("ArKaBankSystem", "ArKaBankLimit", 3000));
}


// ARKABANK_NAME object created on top after includes

extern "C" __declspec(dllexport) ArKaBankClass * Plugin_Init()
{
	return &ARKBANK_NAME;
}
