/*	Renegade Scripts.dll
    ArKa_47 Bank Plugin
	Copyright 2021 ArKa_47, Shadow-Team

	This file is part of the Renegade scripts.dll
	The Renegade scripts.dll is free software; you can redistribute it and/or modify it under
	the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2, or (at your option) any later
	version. See the file COPYING for more details.
	In addition, an exemption is given to allow Run Time Dynamic Linking of this code with any closed source module that does not contain code covered by this licence.
	Only the source code to the module(s) containing the licenced code has to be released.
*/

#include "General.h"
#include "gmplugin.h"
#include "da.h"
#include "da_settings.h"



#include "ArKaBankSystem.h"




 // ********************************************** BANK ENTRY *****************************************************************************************
// BankEntryClass - Constructor -------------------------
ArKaBankEntryClass::ArKaBankEntryClass(const char* playerName, const char* playerHash, const char* stringBalance) {
	this->playerName = playerName;
	this->playerHash = playerHash;
	this->Set_Balance(stringBalance);
}

void ArKaBankEntryClass::Set_Balance(const char* stringBalance) {

	this->stringBalance = stringBalance;
}


// BankEntryClass Getters ---------------------------
const StringClass ArKaBankEntryClass::Get_Name() {
	return this->playerName;
}

const StringClass  ArKaBankEntryClass::Get_Player_Hash() {
	return this->playerHash;
}

const StringClass ArKaBankEntryClass::Get_Balance() {
	return this->stringBalance;
}



// BankEntryClass - Matches methods --------------------
bool ArKaBankEntryClass::Matches_Name(const char* playerName) {
	return (this->playerName == playerName);
}

bool ArKaBankEntryClass::Matches_PlayerHash(const char* playerHash) {
	return (this->playerHash == playerHash);
}

// ArKaBankList - Get Count ------------------------------
int ArKaBankListClass::Get_Count() {
	return this->List.Count();
}

 // ***************************************** BANK LIST ****************************************************************

// BankListClass - Add an entry --------------------------
ArKaBankEntryClass* ArKaBankListClass::Add_Entry(const char* playerName, const char* playerHash, const char* stringBalance) {
	ArKaBankEntryClass* Entry = new ArKaBankEntryClass(playerName, playerHash, stringBalance);
	this->List.Add(Entry);

	return Entry;
}


// BankListClass - Adds an entry by using a single string. Name, Hash and balance: NAME\tHASH\tBALANCE
ArKaBankEntryClass* ArKaBankListClass::Add_Entry(StringClass Format) {

	const char* playerName, * playerHash, * stringBalance;

	DATokenParserClass Parser(Format, '\t');
	playerName = Parser.Get_String();
	playerHash = Parser.Get_String();
	stringBalance = Parser.Get_String();

	// Console_Output("Name: %s, Hash: %s, Balance: %s \n", playerName, playerHash, stringBalance); // For Debug only
	ArKaBankEntryClass* Entry = ArKaBankListClass::Add_Entry(playerName, playerHash, stringBalance);

	return Entry;

}

// BankListClass - Find Entry by Name (Not Safe if different players use the same name [name spoofing?])
ArKaBankEntryClass* ArKaBankListClass::Find_Entry_by_Name(const char* playerName) {
	for (int i = 0; i < this->List.Count(); i++) {
		if (this->List[i]->Matches_Name(playerName)) {
			return this->List[i];
		}
	}
	return 0;
}

// BankListClass - Find Entry by PlayerHash (safer because serial is not that easy to spoof a certain player)
ArKaBankEntryClass* ArKaBankListClass::Find_Entry_by_PlayerHash(const char* playerHash) {
	for (int i = 0; i < this->List.Count(); i++) {
		if (this->List[i]->Matches_PlayerHash(playerHash)) {
			return this->List[i];
		}
	}
	return 0;
}

// BankListClass - Remove Entry ---------------------------------
void ArKaBankListClass::Remove_Entry(const char* playerHash) {
	for (int i = 0; i < this->List.Count(); i++) {
		if (this->List[i]->Matches_PlayerHash(playerHash)) {
			delete List[i];
			List.Delete(i);
		}
	}
}



// BankListClass - Constructor ---------------------------------
ArKaBankListClass::ArKaBankListClass(const char* FileName) {
	this->MaxAmount = 3000; // default value for upper limit
	// Get Limit Value by *.ini - File
	// 
	// 

	/*   */

	DASettingsManager::Add_Settings("arka_bank.ini");







	this->FileName.Format("%s%s", Get_File_Path(), FileName);
	this->LastModTime = 0;
	this->Load();
}

// BankListClass - Kinda default constructor calls the regular construcor
ArKaBankListClass::ArKaBankListClass() : ArKaBankListClass("ArKa_bank.tsv") { }



// BankListClass - destructor ----------------------------------
ArKaBankListClass::~ArKaBankListClass() {

	DASettingsManager::Remove_Settings("arka_bank.ini");

	Save();
	for (int i = 0; i < List.Count(); i++) {
		delete List[i];
	}
	List.Delete_All();
}


// BankListClass - Load the file data into list ---------------
void ArKaBankListClass::Load() {

	TextFileClass File;
	if (File.Open(FileName, 1) && File.Get_Date_Time() != LastModTime) {
		for (int i = 0; i < List.Count(); i++) {
			delete List[i];
		}
		List.Delete_All();
		LastModTime = File.Get_Date_Time();
		StringClass Buffer;
		bool Looping = true;
		while (File.Read_Line(Buffer) || Looping) { // Read_Line gets me always a false.. idk y ; thats y I use the boolean Loop thing
			DATokenParserClass Parser(Buffer, '\t');
			const char* playerName = Parser.Get_String();
			const char* playerHash = Parser.Get_String();
			if (playerName && playerHash) {
				List.Add(new ArKaBankEntryClass(playerName, playerHash, Parser.Get_String()));
			}
			else Looping = false;
		}
		File.Close();
	}
	Load_Settings();
}


// BankListClass - Save the data into the given file ----------------
void ArKaBankListClass::Save() {

	Delete_Empty_Accounts();

	TextFileClass File;

	if (File.Open(FileName, 2)) {

		StringClass Buffer;
		for (int i = 0; i < List.Count(); i++) {
			Buffer.Format("%s\t%s\t%s", List[i]->Get_Name(), List[i]->Get_Player_Hash(), List[i]->Get_Balance());
			File.Write_Line(Buffer);
		}
		LastModTime = File.Get_Date_Time();
	}
	File.Close();
}


// BankListClass - Update Balance --------------------------------------
void ArKaBankListClass::Update_Balance(const char* playerHash, const char* stringBalance) {
	for (int i = 0; i < List.Count(); i++) {
		if (List[i]->Matches_PlayerHash(playerHash)) {
			List[i]->Set_Balance(stringBalance);
		}
	}
}



StringClass ArKaBankEntryClass::Get_Hash_by_PlayerName(const char* playerName) {
	return Get_Client_Serial_Hash(Get_Player_ID_By_Name(playerName));
}

//  BankListClass - Check if Hash exist in List --------------------------
bool ArKaBankListClass::Hash_Exist(const char* playerHash) {
	for (int i = 0; i < List.Count(); i++) {
		if (List[i]->Matches_PlayerHash(playerHash)) {
			return true;
		}
	}
	return false;
}


// BankListClass - Add Money ----------------------------------------------
void ArKaBankListClass::Add_Money(cPlayer* Player, float money) {

	float playerMoney = Player->Get_Money();
	if (playerMoney < money)
		DA::Page_Player(Player->Get_Id(), "You don't have enough credits!"); // if player don't have the credits it's placing
	else {
		if (money >= 0) {



			StringClass playerName, playerHash;
			ArKaBankEntryClass* Entry = NULL;
			float maxDeposit = 0.0f;
			float newBalance = 0.0f;


			playerHash = Get_Client_Serial_Hash(Player->Get_Id());
			Entry = Find_Entry_by_PlayerHash(playerHash);

			// if player exist in List
			if (Entry) {
				float playerBalance = StringToFloat(Entry->Get_Balance());
				/* problem: if playerBalance > MaxAmount... eg. if the MaxAmount got changed  */
				if (MaxAmount < playerBalance){
					DA::Page_Player(Player->Get_Id(), "Your balance of %.0f credits is over the currently limit of %.0f credits. ", playerBalance, MaxAmount);
					maxDeposit = 0;
				}
				else
				maxDeposit = MaxAmount - playerBalance;

				// if player's deposit is too high
				if (money > maxDeposit) {
					money = maxDeposit;
					newBalance = money + playerBalance;
					DA::Page_Player(Player->Get_Id(), "Your maximum deposit of %.0f credits will be made", money); // Inform the player what deposit can be made
					Entry->Set_Balance(StringClass::getFormattedString("%.0f", newBalance));
					Player->Set_Money(playerMoney - money);
					DA::Page_Player(Player->Get_Id(), "Your new account balance is: %.0f credits", StringToFloat(Entry->Get_Balance()));
				}
				// if player's deposit fits
				else {
					newBalance = money + playerBalance;
					DA::Page_Player(Player->Get_Id(), "Your deposit of %.0f credits will be made", money); // Inform the player what deposit can be made
					Entry->Set_Balance(StringClass::getFormattedString("%.0f", newBalance));
					Player->Set_Money(playerMoney - money);
					DA::Page_Player(Player->Get_Id(), "Your new account balance is: %.0f credits", StringToFloat(Entry->Get_Balance()));
				}

			}
			// if player don't exist in List
			else {

				maxDeposit = MaxAmount;
				StringClass entryData;

				// if player's deposit is too high
				if (money > maxDeposit) {

					money = maxDeposit;
					newBalance = money;
					DA::Page_Player(Player->Get_Id(), "Your maximum deposit of %.0f credits will be made", money); // Inform the player what deposit can be made
					entryData.Format("%s\t%s\t%.0f", Get_Player_Name_By_ID(Player->Get_Id()), playerHash, newBalance);
					Entry = Add_Entry(entryData);
					Player->Set_Money(playerMoney - money);
					DA::Page_Player(Player->Get_Id(), "Your new account balance is: %.0f credits", StringToFloat(Entry->Get_Balance()));
				}
				// if player's deposit fits
				else {

					newBalance = money;
					DA::Page_Player(Player->Get_Id(), "Your deposit of %.0f credits will be made", money); // Inform the player what deposit can be made
					entryData.Format("%s\t%s\t%.0f", Get_Player_Name_By_ID(Player->Get_Id()), playerHash, newBalance);
					Entry = Add_Entry(entryData);
					Player->Set_Money(playerMoney - money);
					DA::Page_Player(Player->Get_Id(), "Your new account balance is: %.0f credits", StringToFloat(Entry->Get_Balance()));
				}

			}

		}

		else
			DA::Page_Player(Player->Get_Id(), "Nice try ;)"); // msg to inform that negative money can't be added ; use Remonve_Money instead?
	//	this->Save(); // for debug atm
	}

}


// BankListClass - Remove Money ----------------------------------------------
void ArKaBankListClass::Remove_Money(cPlayer* Player, float money) {


	float playerMoney = Player->Get_Money();

	if (money >= 0) {

		StringClass playerName, playerHash;
		playerHash = Get_Client_Serial_Hash(Player->Get_Id());
		ArKaBankEntryClass* Entry = NULL;
		//float maxWithdrawal = 0.0f;
		float newBalance = 0.0f;


		Entry = Find_Entry_by_PlayerHash(playerHash);

		// if player exist in list
		if (Entry) {
			float playerBalance = StringToFloat(Entry->Get_Balance());

			// if player's withdrawal is too high
			if (money > playerBalance) {
				money = playerBalance;
				if (playerBalance < 1 && playerBalance >= 0)
					DA::Page_Player(Player->Get_Id(), "Your balance is 0 credits");
				else {
					DA::Page_Player(Player->Get_Id(), "Your maximum withdrawal of %.0f credits will be made", money); // Inform the player what withdrawal can be made
					Entry->Set_Balance(StringClass::getFormattedString("%.0f", 0.0f));
					Player->Set_Money(playerMoney + money);
					DA::Page_Player(Player->Get_Id(), "Your new account balance is: %.0f credits", StringToFloat(Entry->Get_Balance()));
				}
			}
			// if player's withdrawal fits
			else {

				newBalance = playerBalance - money;
				DA::Page_Player(Player->Get_Id(), "Your withdrawal of %.0f credits will be made", money); // Inform the player what withdrawal can be made
				Entry->Set_Balance(StringClass::getFormattedString("%.0f", newBalance));
				Player->Set_Money(playerMoney + money);
				DA::Page_Player(Player->Get_Id(), "Your new account balance is: %.0f credits", StringToFloat(Entry->Get_Balance()));

			}

		}
		// if player don't exist in list
		else {
			DA::Page_Player(Player->Get_Id(), "You don't have an account yet. For more information type: !bank info");
		}
	}
	else
		DA::Page_Player(Player->Get_Id(), "Nice try ;)"); // msg to inform that negative money can't be removed ; use Add_Money instead?
	// this->Save(); // for debug atm
}


// BankListClass  - Delte_Empty_Accounts ----------------------------------------
void ArKaBankListClass::Delete_Empty_Accounts() {

	int money;

	

	for (int i = 0; i < List.Count(); i++) {
		money = (int)StringToFloat(List[i]->Get_Balance());
		if ( money == 0 ) {
			delete List[i];
			List.Delete(i);
		}
	}
	
	// Save();

}

// BankListClass -  String To Float ----------------------------------------------
float ArKaBankListClass::StringToFloat(const char* String) {
	char* end;
	return std::strtof(String, &end);
}


// BankListClass - SetMaxAmount --------------------------------------------------
void ArKaBankListClass::Set_MaxAmount(float maxAmount) {
	this->MaxAmount = maxAmount;
}

// BankListClass - GetMaxAmount --------------------------------------------------
float ArKaBankListClass::Get_MaxAmount() {
	return this->MaxAmount;
}

// BankListzClass - Print_Info- -----------------------------------------------------
void ArKaBankListClass::Print_Info(cPlayer* Player){
	DA::Page_Player(Player->Get_Id(), "use !bank dp|wd [CREDITS] to depose or withdraw credits from bank");
	DA::Page_Player(Player->Get_Id(), "The current limit is: %.0f credits.", MaxAmount);
}

// BankListClass - Print Acount Info -----------------------------------------------
void ArKaBankListClass::Print_AccountInfo(cPlayer* Player){
	ArKaBankEntryClass* Entry = NULL;
	Entry = Find_Entry_by_PlayerHash(Get_Client_Serial_Hash(Player->Get_Id()));
	if (!Entry)
		Print_Info(Player);
	else{
		Print_Info(Player);
		DA::Page_Player(Player->Get_Id(), "Your account balance is: %.0f credits", StringToFloat(Entry->Get_Balance()));


	}
}

// BankListClass - Has Account ---------------------------------------------------------
bool ArKaBankListClass::Has_Account(cPlayer* Player){
	return Hash_Exist(Get_Client_Serial_Hash(Player->Get_Id()));
}

// BankListClass - Get Balance --------------------------------------------------------
float ArKaBankListClass::Get_Balance(cPlayer* Player)
{
	ArKaBankEntryClass* Entry = NULL;

	Entry = Find_Entry_by_PlayerHash(Get_Client_Serial_Hash(Player->Get_Id()));
	if (!Entry)
	return 0.0f;
	else {
		return StringToFloat(Entry->Get_Balance());
	}
}

void ArKaBankListClass::Load_Settings() {



}